## 必要なパッケージをR にインストールしたうえでライブラリに追加する
libraries = c("readxl", "dplyr", "magrittr", "tidyr", "RColorBrewer", "surveillance", "ggplot2")
for(x in libraries) {library(x,character.only=TRUE, warn.conflicts=FALSE,quietly=TRUE)}

## データセットを追加する
read.csv("JapaneseDataCOVID19_Rcode.csv") -> data
data %<>% mutate(onset = as.Date(onset))

## 発症間隔の分布を定義する(Nishiura et al., 2020, IJID)
gi_fit = list(shape=2.305, scale=5.452)
generation <- function(t){pweibull(t, shape = gi_fit$shape, 
                                   scale = gi_fit$scale) - pweibull(t-1, 
                                   shape = gi_fit$shape, scale = gi_fit$scale)}
## 潜伏期間の分布を定義する(Linton et al., 2020, JCM)
inc_fit = list(meanlog=1.519, sdlog=0.615)
incubation <- function(t){plnorm(t, inc_fit$meanlog, 
                                 inc_fit$sdlog) - plnorm(t-1, inc_fit$meanlog, inc_fit$sdlog)}
## 発症から診断までの分布を定義する(Jung et al., 2020)
rep_fit = list(shape=1.741, scale=8.573)
onsettolabconf <- function(t){pweibull(t, shape = rep_fit$shape, 
                                       scale = rep_fit$scale) - pweibull(t-1, shape=rep_fit$shape, scale=rep_fit$scale)}

## 逆計算の安定性のため10日分のNA を追加する
temp_lastdays <- matrix(NA, ncol=1, nrow=10)
temp_lastdays %<>% as.data.frame() %>% 
  mutate(onset=as.Date((max(data$onset)+1):(max(data $onset)+10), origin="1970-01-01"), 
         domestic=0, imported=0, total=0) %>% 
  select(-V1)
rbind(data, temp_lastdays) -> data
data %<>% mutate(time_onset = 1:nrow(data))

## 潜伏期間の分布を定義する
K = nrow(data)
incubation_pdf = plnorm(1:K, inc_fit$meanlog, inc_fit$sdlog) - plnorm(1:K-1, inc_fit$meanlog, inc_fit$sdlog)
inc_pmf = c(0, incubation_pdf [1:21])
## 輸入感染例の逆計算
sts = new("sts", epoch= data$time_onset, observed=data$imported)
bpnp.control = list(k = 2, eps = rep(1e-4,2), 
                    iter.max=rep(1000,2), Tmark = nrow(sts), 
                    B = -1, alpha = 0.01, verbose = FALSE, 
                    lambda0 = NULL, eq3a.method = c("R","C"))
sts_bp = backprojNP(sts, incu.pmf=inc_pmf, control=modifyList(bpnp.control,list(eq3a.method="C")))
data $imported_backproj = upperbound(sts_bp)
## 国内感染例の逆計算
sts = new("sts", epoch= data$time_onset, observed=data$domestic)
bpnp.control = list(k = 2, eps = rep(1e-4,2), iter.max=rep (1000,2), 
                    Tmark = nrow(sts), B = -1, alpha = 0.01, verbose = FALSE, 
                    lambda0 = NULL, eq3a.method = c("R","C"))
sts_bp = backprojNP(sts, incu.pmf=inc_pmf, control=modifyList(bpnp.control,list(eq3a.method="C")))
data $domestic_backproj = upperbound(sts_bp)

## 逆計算した報告数の正規化
data %>% mutate(imported_normal = imported_backproj / sum(imported_backproj) * sum(imported), 
  domestic_normal = domestic_backproj / sum(domestic_backproj) * sum(domestic), 
  total = imported_normal + domestic_normal, time_onset = 0:(nrow(data)-1)) %>%
  select(time_onset, onset, imported_normal, domestic_normal, total) %>%
  rename(t = time_onset, imported = imported_normal, domestic = domestic_normal)　%>%
  filter(onset <= (as.Date("2020-05-10")-1)) -> dt.backproj

##遅れの補正を伴う再生産方程式
today = as.numeric(as.Date("2020-05-10") -as.Date(min(dt.backproj$onset)))
## 2月1日から推定を実施(それ以前の流行初期は報告数が少ないことから不確実性が高いため)
Start.T = 29
## 感染から確定診断までの報告遅れの分布を定義する。
##2021-12-10 読者の指摘があり、以下2行コメントアウト。
##infectiontoreport <- function(t, tau){sum(convolve(onsettolabconf(1:today-t+tau),
##                                                   rev(incubation(1:today-t+tau)),type = c("open")))}
##次の二行が差し替え分（2021-12-10）
infectiontoreport <- function(t, tau){sum(convolve(onsettolabconf(1:1000),
                                                   rev(incubation(1:1000)),type = c("open"))[1:(today-t+tau)])}
## 計算時間を早めるための事前の計算
precal <- matrix(0, nrow=(max(dt.backproj$t)-Start.T+2+1), ncol=(max(dt.backproj$t)-1))
for (m in (Start.T-2):max(dt.backproj$t)){for (n in 1:m-1){precal[m-Start.T+2+1,n] <- infectiontoreport(m,n)}}
delay_precalculation <- function(t){sum(convolve(onsettolabconf(1:(today-t)),
                                                 rev(incubation(1:(today-t))), type = c("open")))}
dt.backproj %<>% rowwise %>% mutate(imported_delay = imported/delay_precalculation(t),
                                    domestic_delay = domestic/delay_precalculation(t),
                                    total = imported_delay+domestic_delay)
##Cori によって提案された瞬間的再生産数の推定
est.t <- list(); est.CI <- list();
for (TT in Start.T:max(dt.backproj$t)){
  dt.backproj %>% 
  filter(t <= TT) -> dt.backproj.T
  llk <- function(param){t=TT; R = param
  llk <- 0; Cs <- 0 ;　Css <- rep(0, t)
  for (tau in 1: t - 1){Css[tau] = (dt.backproj.T$total[t-tau+1]) * generation(tau) / precal[t - Start.T + 3, tau]}
      Cs = sum(Css) * R
      Cs[Cs <= 0] <- 1e-5
      return(-(-Cs+dt.backproj.T$domestic_delay[t+1] * log(Cs) - lgamma(dt.backproj.T$domestic_delay[t + 1] + 1)))
    }
    param0 = c(0.7)
    opt_est <- optim(param0, fn=llk, method=c("L-BFGS-B"), lower=c(0), control=list(maxit = 10000))
    opt_est$par -> est.t[[TT]]
    ## プロファイル尤度による95% 信頼区間の推定
    ci_pro <- matrix(NA, ncol = 2, nrow = 1)
    CI <- function(par_CI){return(2 * (-llk(par_CI)+opt_est$value))} #llk()
    par_CI <- seq(0, 10, by = 0.01)
    logLik <- sapply(par_CI, FUN = CI)
    as.data.frame(par_CI) -> par_CI; as.data.frame
    (logLik) -> logLik
    cbind(par_CI, logLik) -> data_CI
    data_CI$logLik[data_CI$logLik < (max(data_CI$logLik) - 3.84)] <- NA
    data_CI %<>% na.omit()
    min(data_CI$par_CI) -> ci_pro[1,1]; max(data_CI$par_CI) -> ci_pro[1,2]
    as.data.frame(ci_pro) -> ci_pro
    colnames(ci_pro) <- c("lower","upper")
    ci_pro -> est.CI[[TT]]
  }
matrix(unlist(est.t),ncol=1,byrow=T) -> est.t
matrix(unlist(est.CI),ncol=2,byrow=T) -> est.CI
cbind(est.t, est.CI) -> est
  
## 作図のために結果(est)を調整
est %>% as.data.frame() %>% mutate(t = Start.T:max(dt.backproj$t)) -> result
merge(dt.backproj, result, by='t', all.x=TRUE) -> result
colnames(result) <- c("t","onset","imported","domestic", "total","imported_delay","domestic_delay","Rt","lower", "upper")
result %>% filter(onset <= as.Date(as.Date("2020-05-10")-(17))) -> result_withcut
## ggplot2による作図
options(repr.plot.width = 7, repr.plot.height = 5)
theme_set(theme_bw())
scaling_parameter = max(result_withcut$total) / max(result_withcut$upper[!is.na(result_withcut$upper)])
range = c(as.Date("2020-02-01"), (as.Date("2020-05-10") -17))
options(warn=-1)
result_withcut %>% 
    ggplot() +
    geom_bar(aes(x=onset, y=total), stat='identity', fill="gray35", width=0.7) + 
    geom_line(data=result[!is.na(result$Rt),],aes(x=onset,y=Rt * scaling_parameter), color="#1380A1", size=1) +
    geom_ribbon(data=result, aes(ymax=result$upper * scaling_parameter, 
                                 ymin=result$lower * scaling_parameter, x=onset), 
                fill="#1380A1", alpha = 0.4) + labs(x="n 推定感染日", y=" 感染者数n") +
    theme (text = element_text ( size = 12 , family="sans",color = "black"),
           axis.text = element_text (size = 12 , family="sans",color = "black"),
           panel.grid.major=element_blank(), panel.grid.minor = element_blank(),
           legend.position="none") +
    scale_x_date(date_labels="%m/%d", date_breaks="10 day",
                 limits=range, expand=c(0, 0)) +
    scale_y_continuous(limit=c(0,620), expand = c(0, 0),
                       sec.axis=sec_axis(~./(scaling_parameter), breaks=c(0,2,4), name =" 実効再生産数n")) + 
    geom_hline(yintercept=1 * scaling_parameter,
               linetype="dashed", color = "#1380A1", size =0.7)
  